"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: xmlaccessTransactionSettings.py

	Used by	the following actions to handle transaction setting in the wp server
		wp_common_deploy_set_xmlaccess_transaction_settings
		wp_common_deploy_restore_xmlaccess_transaction_settings
"""
import sys
import os
from java.io import FileOutputStream
from java.io import FileInputStream
from java.io import File
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

LOGGER = _Logger("xmlaccessTransactionSettings", MessageManager.RB_WEBSPHERE_WP)

def _getAppServerId(nodeName, serverName):
	"""
	Returns the serverId of the server given by the supplied name on the supplied node
	
	Throws an exception if the serverId cannot be retrieved
	"""
	LOGGER.traceEnter([nodeName, serverName])
	cellName = AdminControl.getCell()
	
	serverId = AdminConfig.getid("/Cell:" + cellName + "/Node:" + nodeName + "/Server:" + serverName + "/")
	if serverId == "":
		raise "Error -- Server " + serverName + " does not exist on node " + nodeName
	#endIf
	type = AdminConfig.showAttribute(serverId, 'serverType')
	if (type == "APPLICATION_SERVER"):
		## a valid app server
		LOGGER.traceExit(serverId)
		return serverId
	else:
		## throw an exception, not an app server
		raise "Error -- Server " + serverName + " is not an Application Server"
	#endIf
#endDef

def _getTransactionServiceId(serverId):
	"""
	Returns the id of the TransactionService contained within the supplied serverId
	
	Throws an exception if the id cannot be retrieved
	"""
	LOGGER.traceEnter([serverId])
	appServer = AdminConfig.list( "ApplicationServer", serverId ).split( newline )[0]
	if (len(appServer) > 0):
		txServiceId = AdminConfig.list( "TransactionService", appServer ).split( newline )[0]
		if (len(txServiceId) > 0):
			LOGGER.traceExit(txServiceId)
			return txServiceId
		#endIf
	#endIf
	raise "Error -- Unable to find TransactionService as a child of serverId: " + str(serverId)
#endDef

def _updateTransactionSettings(serverId, attributes):
	"""
	Updates transaction settings for the supplied serverId
	
	attributes is a list of [name,value] pairs to supply to AdminConfig
	"""
	LOGGER.traceEnter([serverId, attributes])
		
	txService = _getTransactionServiceId(serverId)
	AdminConfig.modify(txService, attributes)
	LOGGER.traceExit()
#endDef

def _getTransactionSettings(serverId):
		
	"""
	Returns the current transaction settings for the supplied serverId
	as a dictionary.	
	"""
	LOGGER.traceEnter([serverId])
	txService = _getTransactionServiceId(serverId)
	lifetimeTimeout = AdminConfig.showAttribute(txService, 'totalTranLifetimeTimeout')
	maxTimeout = AdminConfig.showAttribute(txService, 'propogatedOrBMTTranLifetimeTimeout')
	attributes = {}
	attributes["lifetimeTimeout"] = lifetimeTimeout
	attributes["maxTimeout"] = maxTimeout
	LOGGER.traceExit(attributes)
	return attributes

#endDef

## the first line in the xmlaccess.properties file that is written by this script
HEADER="Defines server settings used to support xmlaccess; especially transaction-level=\"request\""
# parse the options into optDict
optDict, args = SystemUtils.getopt( sys.argv, 'mode:;nodeName:;serverName:;properties:;action:')
mode = optDict['mode']
nodeName = optDict['nodeName']
serverName = optDict['serverName']
properties = optDict['properties']
action = optDict['action']

if (mode == MODE_EXECUTE):
	propsFile = File(properties)
	props = Properties()
	if (propsFile.exists()):
		props.load(FileInputStream(propsFile))
	else:
		## set default values so that they will get saved when the props are saved
		props.setProperty("lifetimeTimeout", "1200")
		props.setProperty("maxTimeout", "9000")
	#endIf
		
	serverId = _getAppServerId(nodeName, serverName)	
	if (action == "SET"):
		## read the current settings and save to file
		currentSettings = _getTransactionSettings(serverId)
		props.setProperty("previous.lifetimeTimeout", currentSettings["lifetimeTimeout"])
		props.setProperty("previous.maxTimeout", currentSettings["maxTimeout"])
		
		LOGGER.log("CRWWP0001I", [currentSettings["lifetimeTimeout"], currentSettings["maxTimeout"], properties])
		
		## update with new settings
		lifetimeTimeout = props.getProperty("lifetimeTimeout", "1200")
		maxTimeout = props.getProperty("maxTimeout", "9000")
		LOGGER.log("CRWWP0002I", [lifetimeTimeout, maxTimeout, serverName])
		attributes = [['totalTranLifetimeTimeout', lifetimeTimeout], ['propogatedOrBMTTranLifetimeTimeout', maxTimeout ]]
		_updateTransactionSettings(serverId, attributes)
		
		## save the previous values
		props.store(FileOutputStream(properties), HEADER)
	else:
		## action is to reverse
		## restore to previous settings
		lifetimeTimeout = props.getProperty("previous.lifetimeTimeout")
		maxTimeout = props.getProperty("previous.maxTimeout")
		if (lifetimeTimeout is not None and maxTimeout is not None):
			LOGGER.log("CRWWP0003I", [lifetimeTimeout, maxTimeout, serverName])
			attributes = [['totalTranLifetimeTimeout', lifetimeTimeout], ['propogatedOrBMTTranLifetimeTimeout', maxTimeout ]]
			_updateTransactionSettings(serverId, attributes)
		else:
			LOGGER.log("CRWWP0004E", [lifetimeTimeout, maxTimeout, properties])
			raise "Error -- Unable to restore transaction service settings; the previous values are not available."
		#endIf	
	#endIf	
	AdminHelper.saveAndSyncCell()
else:
	print "Unsupported MODE supplied: " + str(mode)