"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigValidator.py
	
	This contains methods and classes to facilitate validating input 
	before the WebSphere configuration data is modified.
"""

import AdminConfig
import java

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from AdminHelper import AdminHelper
from ConfigReader import ConfigReader
from ConfigReader import ConfigTypeKeys
from XmlProperty import InvalidXmlConfigException
from SystemUtils import SystemUtils

newline = java.lang.System.getProperty("line.separator")

class ConfigValidator:
	
	def __init__(self):
		self.LOGGER = _Logger("ConfigValidator", MessageManager.RB_WEBSPHERE_WAS)
		self.uniqueAttributeName = "name"
		self.configTypeKeys = ConfigTypeKeys()
	#endDef

	"""
	Returns the name of the attribute that should be unique 
	across multiple elements of the same configuration type.
	
	Defaults to 'name'	
	"""
	def getUniqueAttributeName(self):
		return self.uniqueAttributeName
	#endDef
	
	"""
		Validates the configuration in xmlNode can be created under 
		the scope given by parentId
		
		This method can be overridden to provide custom validation 
		for specific WebSphere types.
		
		If the data is invalid, returns 0
		If the data is valid,   returns 1
		
		@raise InvalidXmlConfigException: If the supplied xmlNode contains invalid data. 
	"""
	def validate(self, xmlNode, parentId):
		return self.validateUniqueAttr(xmlNode, parentId, self.getUniqueAttributeName())
	#endDef
	
	"""
		Same as validate except return value
		If the data is not unique, returns the child id that must be updated
		If the data is unique, return None
	"""	
	def validate2(self, xmlNode, parentId):
		return self.validateUniqueAttr2(xmlNode, parentId, self.getUniqueAttributeName())
	#endDef

	"""
		Validates the configuration in xmlNode can be created under 
		the scope given by parentId
		
		This method uses jndiName as the attribute name which must be unique
		
		This method can be overridden to provide custom validation 
		for specific WebSphere types.
		
		If the data is invalid, returns 0
		If the data is valid,   returns 1
		
		@raise InvalidXmlConfigException: If the supplied xmlNode contains invalid data. 
	"""
	def validateUniqueJndiName(self, xmlNode, parentId):
		attrName = "jndiName"
		return self.validateUniqueAttr(xmlNode, parentId, attrName)
	#endDef	

	"""
	Same as validateUniqueJndiName except return value
		If the data is not unique, returns the child id that must be updated
		If the data is unique, return None
	"""	
	def validateUniqueJndiName2(self, xmlNode, parentId):
		attrName = "jndiName"
		return self.validateUniqueAttr2(xmlNode, parentId, attrName)
	#endDef
	
	"""
		Validates the configuration in xmlNode does not contain
		a match in the WebSphere cell for the type, parentId 
		and attribute name supplied
		
		If the data is invalid, returns 0
		If the data is valid, returns 1
		
		@raise InvalidXmlConfigException: If the supplied xmlNode contains invalid data. 
	"""
	def validateUniqueAttr(self, xmlNode, parentId, attrName):
		childId = self.validateUniqueAttr2(xmlNode, parentId, attrName)
		if (childId is None):
			return 1
		else:
			return 0
		#endIf
	#endDef
	
	"""
		Same as validateUniqueAttr except return value
		If the data is not unique, returns the child id that must be updated
		If the data is unique, return None
	"""	
	def validateUniqueAttr2(self, xmlNode, parentId, attrName):
		type = xmlNode.getNodeNameFixed()
		parentScope = AdminHelper.getScope(parentId)
		if (xmlNode.hasAttr(attrName)):
			rafwName = xmlNode.getAttrValue(attrName)
			for childId in AdminConfig.list(type, parentId).split(newline):
				if (len(childId) > 0):
					## at cell scope, AdminConfig.list will return objects at node/server/clusters scopes
					## which will skue the results, only consider this object as not unique if it can be 
					## found at the current scope
					if (AdminHelper.isInScope(parentScope, childId)):
						wasConfigName = AdminConfig.showAttribute(childId, attrName)
						if (rafwName == wasConfigName):
							if (not SystemUtils.updateOnAugment()):
								print "Warning: An object of type " + type + " with " + attrName 
								print "         '" + wasConfigName + "' already exists."
								print "         You must use a unique value for " + attrName 
								print "         when creating objects of type " + type;
							#endIf
							return childId
						#endIf
					else:
						self.LOGGER.debug("Discovered object which is not in the same scope: " + parentScope + " objectId: " + childId);
					#endIf
				else:
					self.LOGGER.info("No children of type: " + type + " can be found with parent: " + parentId);
			#endFor
		else: 
			msg = ("Supplied xml node doesn't contain the attribute '" + str(attrName) + "' " +
			       "This data cannot be validated before creating the configuration in WebSphere. xmlNode=" + str(xmlNode))
			self.LOGGER.error(msg)
			raise InvalidXmlConfigException(msg)
		#endIf
		return None
	#endDef
	
	"""
		Validates the configuration in xmlNode does not contain
		a match in the WebSphere cell for the type, parentId 
		and attribute names supplied
		
		attrNames is a list of attribute names, all of which must match
		to be considered not unique
		
		If attrNames is None or not supplied, attribute names which match the 
		ConfigType in ConfigTypeKeys are used.
		
		If the data is invalid, returns 0
		If the data is valid, returns 1
		
		@raise InvalidXmlConfigException: If the supplied xmlNode contains invalid data. 
	"""
	def validateUniqueAttrs(self, xmlNode, parentId, attrNames = None):
		childId = self.validateUniqueAttrs2(xmlNode, parentId, attrNames)
		if (childId is None):
			return 1
		else:
			return 0
		#endIf
	#endDef
	
	"""
		Same as validateUniqueAttrs except return value
		If the data is not unique, returns the child id that must be updated
		If the data is unique, return None
	"""	
	def validateUniqueAttrs2(self, xmlNode, parentId, attrNames = None):
		self.LOGGER.traceEnter([xmlNode, parentId, attrNames])
		type = xmlNode.getNodeNameFixed()
		if (attrNames is None):
			attrNames = self.configTypeKeys.getUniqueAttrs(type)
		if (xmlNode.hasAllAttrs(attrNames)):
			rafwNames = xmlNode.getAttrValues(attrNames)
			reader = ConfigReader()
			for childId in AdminConfig.list(type, parentId).split(newline):
				if (len(childId) > 0):
					wasConfigNames = reader.show(childId)
					## see if all attrNames match
					allAttrsMatch = 1
					for attrName in attrNames:
						if (wasConfigNames.has_key(attrName)):
							if (wasConfigNames[attrName] == rafwNames[attrName]):
								self.LOGGER.trace("values are the same for " + attrName + " in RAFW and WAS configuration");
							else:
								self.LOGGER.trace("values differ for " + attrName + " in RAFW and WAS configuration");
								self.LOGGER.trace("   " + str(wasConfigNames[attrName]) + " != " + str(rafwNames[attrName]));
								allAttrsMatch = 0
						else:
							self.LOGGER.error("WAS configuration for object: " + childId
											+ " does not have a value for attribute: " + attrName)
							allAttrsMatch = 0
						#endIf
					#endFor		
					if (allAttrsMatch):
						if (not SystemUtils.updateOnAugment()):
							print "Warning: An object of type " + type + " with the same values for "
							print "         attributes: " +  str(attrNames) + " already exists."
							print "         You must use unique values for these attributes"
							print "         when creating objects of type " + type;
						#endIf
						self.LOGGER.traceExit(childId)
						return childId
					#endIf
				else:
					self.LOGGER.info("No children of type: " + type + " can be found with parent: " + parentId);
			#endFor
		else:
			msg= ("Supplied xml node doesn't contain the attributes " + str(attrNames) +
			      " This data cannot be validated before creating the configuration " +
			      "in WebSphere. xmlNode=" + str(xmlNode))
			self.LOGGER.error(msg)
			raise InvalidXmlConfigException(msg)
		#endIf
		self.LOGGER.traceExit(None)
		return None
	#endDef
#endClass
