"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


	File name: J2CConnFactories.py

  This script is to create J2C Connection Factories under Resource Adapters in WAS 
	conneciton factories in a given scope.

  This script is invoked as:
  
     wsadmin -lang jython -profile jythonLib.py -f J2CConnFactories.py 
			-version <WAS version> -scope ${SCOPE} -scopename <scope name> 
			-properties <J2C properties file> 

  The script expects the following parameters:
    -version <WAS version>: specify the WAS version of the server
    -scope <scope>: specify the scope of the j2c connection factories to be created
    -scopename <scope name>: specify the scope name of the jms connection to be created 
    -properties <J2C properties file>: specify the name of the J2C properties 
			file to be used for the creation
"""
from ConfigWriter import ConfigWriter
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from XmlProperty import InvalidXmlConfigException
from AdminHelper import AdminHelper
import java
import AdminConfig


newline = java.lang.System.getProperty("line.separator")

class J2CConnFactoryMediator:
	
	def __init__(self):
		self.myConfigWriter = ConfigWriter();
		self.LOGGER = _Logger("J2CConfigMediator", MessageManager.RB_WEBSPHERE_WAS)
	#endDef
	
	def createConfigType(self, scopeId, scopeType, scope, xmlProp, typeName):
	
		self.myConfigWriter.removeExistingConfig(scopeType, typeName, scopeId)
	
		nodeArray = xmlProp.getFilteredNodeArray(typeName)
		for xmlNode in nodeArray:
			
			## figure out the name and ID of the parent J2C resource adapter
			resourceAdapter = xmlNode.getFilteredChildrenArray("J2CResourceAdapter")
			provider = xmlNode.getFilteredChildrenArray("builtin")
			J2CResourceAdapterName = ""
			
			if ( len(resourceAdapter) > 0 ):
				# there is the potential for multiple adapters within the WAS data model
				# however, I have never seen it used, and it is not yet implemented within RAFW
				J2CResourceAdapterName = resourceAdapter[0].getAttrValue("name")
			elif ( len(provider) == 1 ):
				# there should only be one builtin for CMP connection factories
				J2CResourceAdapterName = provider[0].getAttrValue("name")
			else:
				#raise exception, every connection factory should have an adapter
				raise InvalidXmlConfigException(xmlNode.getAttrValue("name") + 
					" does not contain a J2CResourceAdapter node")
			
			#TODO: refactor to use AdminHelper.findProviderId
			parentId = AdminHelper.findProviderIdByName(scope, 'J2CResourceAdapter',
							J2CResourceAdapterName)[0]
			self.createJ2CWASObject(xmlNode, parentId, scopeId)
		#endFor
		return nodeArray
	#endDef
	
	"""
		Determines the object id for the <ConnectionDefinition> child of the supplied xmlNode
		
		@raise InvalidXmlConfigException: If the <ConnectionDefinition> node cannot be found
		@raise ConfigWriterException: If the reference id cannot be found.  
	"""
	def _getConnectionDefId(self, xmlNode, scopeId, parentId):
		self.LOGGER.traceEnter([xmlNode, scopeId])
		
		connDefNodes = xmlNode.getFilteredChildrenArray("ConnectionDefinition")
		
		# we need to get the connectionDefinition Id from the config to be able to 
		# create the conn factory
		xmlChildren = xmlNode.getFilteredChildrenArray("ConnectionDefinition")
		if ( len(xmlChildren) < 1 ):
			msg = "XML node ConnectionDefinition does not exist as a child of J2CConnectionFactory"
			raise InvalidXmlConfigException(msg)
		#endIf
		
		for xmlChild in xmlChildren:
			childConnFacInt=xmlChild.getAttrValue("connectionInterface")
			conDefs=AdminConfig.list("ConnectionDefinition", parentId).split(newline)
			for conDef in conDefs:
				interfaceType=AdminConfig.showAttribute(conDef, "connectionInterface")
				if (interfaceType == childConnFacInt):
					self.LOGGER.traceExit(conDef)
					return conDef
				#endIf
			#endFor
		#endFor
		
		self.LOGGER.error("Unable to find object ID for " 
			+ str(connDefNodes[0]) + " in scope: " + str(scopeId))
		self.LOGGER.traceExit(conDef)
		#endIf
	#endDef
	
	def createJ2CWASObject(self, xmlNode, parentId, scopeId, excludedNodeNames = ['builtin']):
		self.LOGGER.traceEnter([xmlNode,parentId,scopeId,excludedNodeNames])
		## connectionDefinition is required for types J2CConnectionFactory
		## if the ConnectionDefinition child node exists
		## but not for CMPConnectorFactory
		connDefNodes = xmlNode.getFilteredChildrenArray("ConnectionDefinition")			
		if (len(connDefNodes) > 0 and connDefNodes[0] != None):
			connectionDefId = self._getConnectionDefId(xmlNode, scopeId, parentId)
			xmlNode.setAttrValue("connectionDefinition", connectionDefId)
		#endIf
		newID = self.myConfigWriter.createWASObject(xmlNode, parentId)
		self.LOGGER.traceExit(newID)
		return newID
	#endDef
#endClass