"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: WasDeploy.py
	
	Encapsulates useful utilities used for WebSphere specific application deployment
"""

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from SystemUtils import SystemUtils
from JavaConversionHelper import JavaConversionHelper
from ConfigWriter import ConfigWriterException

import AdminConfig
import AdminControl
import AdminApp

import sys
from ConfigReader import newline

"""
This options filter is used by default in the WasDeploy class to 
filter out options.  Implementors can supply a custom options filter
to filter options.
"""
class NoOpOptionsFilter:
	
	"""
		Includes all options, always returns true
		
		name is the full property name
		value is the property value
		wasOption is the WAS specific name of the option
		
		Return 1(true) to include the option
		Return 0(false) to exclude the option
	"""
	def includeOption(self, name, value, wasOption):
		return 1
	#endDef
	
#endClass

"""
CLASS :: _WasDeploy
Encapsulates useful utilities used for				 
WebSphere specific application deployment				
"""
class _WasDeploy:
	
	def __init__(self):
		self.LOGGER = _Logger("WasDeploy", MessageManager.RB_WEBSPHERE_WAS)
	#endDef
	
	"""
		Based on the contents of the propsFile, creates an options string
		which is used to deploy the application to WAS
	"""
	def createOptions(self, propsFile):
		props = SystemUtils.getPropertyDict(propsFile)
		return self.createOptionsFromDict(props, "")
	#endDef
	
	"""
		Based on the contents of the supplied props dictionary,
		creates an options string which is used to deploy or update
		the application in WAS
	"""
	def createOptionsFromDict(self, props, keyPrefix="", optionsFilter = NoOpOptionsFilter()):
		optionsDict = {}
		optionsStr = ""
		for prop in props.keys():
			if (prop.find(keyPrefix + "options.multi.") > -1):
				# key starts with options.multi
				# process as array
				# keys will be index.name, so we parse off the index portion
				# options.multi. is 14 chars in length
				prefixLength = 14 + len(keyPrefix)
				indexKey = prop[prefixLength:len(prop)]
				value = props[prop].strip()
				wasOption = indexKey.split('.',1)[1]
				if (optionsFilter.includeOption(prop, value, wasOption)):
					key = "-" + wasOption
					if (not optionsDict.has_key(key)):
						optionsDict[key] = []
					#endIf
					optionsDict[key].extend(str(value))
				else:
					self.LOGGER.info("optionsFilter filtered and skipped key: " + prop + " with value: " + value)
				#endIf
			elif (prop.find(keyPrefix + "options.") > -1):
				# key starts with options
				# process as single value
				# options. is 8 chars in length
				prefixLength = 8 + len(keyPrefix)
				key = prop[prefixLength:len(prop)]
				value = props[prop].strip()
				if (optionsFilter.includeOption(prop, value, key)):
					if (self._isBoolean(value)):
						if (value.lower() == "true" or value.lower() == "yes"):
							optionsStr += "-" + key + " "
						else:
							optionsStr += "-no" + key + " "
						#endIf
					else:
						optionsStr += "-" + key + " " + value + " "
					#endIf
				else:
					self.LOGGER.info("optionsFilter filtered and skipped key: " + prop + " with value: " + value)
				#endIf
			#endIf
		#endFor
		## optionsDict contains []s for values, append those to the options string
		for key in optionsDict.keys():
			optionsStr += key + " ["
			for value in optionsDict[key]:
				optionsStr += value
			optionsStr += "] "
		#endFor	
		return optionsStr
	#endDef
	
	"""
		Returns true if the supplied string represents a boolean.
		
		A string represents a boolean if its value is true|false|yes|no ignoring case
	"""
	def _isBoolean(self, value):
		lowerVal = value.lower()
		if (lowerVal == "true" or lowerVal == "false" or lowerVal == "yes" or lowerVal == "no"):
			return 1
		else:
			return 0
	#endDef
	
	"""
	checkIfAppExists 
		Description:									 
			Return true if the application exists (has been deployed)
			to this WebSphere environment
								                            
		Inputs:                                              
			appName = the name of the application in WebSphere
	"""
	def checkIfAppExists (self, appName ):
		appExists = 1
		application = AdminConfig.getid("/Deployment:"+appName+"/" ).strip()
		if (len(application) == 0):
			print "Info: Application with name " + str(appName) + " does not exist in Application Server configuration" 
			appExists = 0
		#endIf 
		return appExists
	#endDef 
	
	def postInstallConfig ( self, appName, warClassLoaderPolicy, classLoaderMode ):
		application = AdminConfig.getid("/Deployment:"+appName+"/" ).strip()
		if (len(application) > 0):
			deployedObject = AdminConfig.showAttribute(application, "deployedObject" )
			classloader = AdminConfig.showAttribute(deployedObject, 'classloader')

			if (warClassLoaderPolicy is None or warClassLoaderPolicy == "SINGLE"):
				AdminConfig.modify(deployedObject, [["warClassLoaderPolicy", "SINGLE"]] )
			else:
				AdminConfig.modify(deployedObject, [["warClassLoaderPolicy", "MULTIPLE"]] )
			#endElse 
			if (classLoaderMode is not None and classLoaderMode == "PARENT_LAST"):
				AdminConfig.modify(classloader, [["mode", "PARENT_LAST"]] )
			else:
				AdminConfig.modify(classloader, [["mode", "PARENT_FIRST"]] )
			#endIf
		else:
			msg = "Application with name " + str(appName) + " does not exist in Application Server configuration" 
			self.LOGGER.error(msg)
			raise ConfigWriterException(msg)
		#endIf
	#endDef 
	
	def installApp ( self, options, scopeArguments, appFile, appName):
		#available installable apps
		appExists = self.checkIfAppExists(appName )
		if (appExists):
			# update the app
			print "Updating "+appName+" ..."
			optionsAsSequence = JavaConversionHelper.convertStringToSequence('-update -appname "'+appName+'" -verbose ' + scopeArguments + ' '+options)
			AdminApp.install(appFile, optionsAsSequence)
		else:
			# install the apps
			print "Installing "+appName+" ..."
			optionsAsSequence = JavaConversionHelper.convertStringToSequence('-appname "'+appName+'" -verbose ' + scopeArguments + ' '+options)
			AdminApp.install(appFile, optionsAsSequence)
		#endElse 
	#endDef 
	
	def startApplication (self, appName, nodeName, serverName ):
		self.LOGGER.traceEnter([appName, nodeName, serverName])	

		print "StartApplication: appName="+appName+" on node "+nodeName+" for server "+serverName+" ..."	    
		appMgrID = AdminControl.queryNames("type=ApplicationManager,node="+nodeName+",process="+serverName+",*" )	    
		length = len(appMgrID)
		self.LOGGER.debug("StartApplication: appMgrID.length="+str(length)+" appMgrID="+appMgrID)
		if (length >= 1):
			try:
				print "startApplication: starting "+appName+"  ..."
				started = AdminControl.invoke(appMgrID, "startApplication", '"' + appName + '"' )
				if (len(started) > 0):
					print started
				#endIf 
			except:
				print "startApplication: Exception trying to start "+appName+" on node "+nodeName+" for server "+serverName
				print " Caused by: " + str(sys.exc_info()[1])
				self.LOGGER.traceExit()
				return 
			#endTry 
		else:			
			self.LOGGER.log("CRWWA1005E", [str(appName), str(serverName)])
			self.LOGGER.traceExit()
			return
		#endIf 
		self.checkApplicationRunning(nodeName, serverName, appName )
		self.LOGGER.traceExit()
	#endDef 
	
	def stopApplication (self, appName, nodeName, serverName ):
		self.LOGGER.traceEnter([appName, nodeName, serverName])	
		print "StopApplication: appName="+appName+" nodeName="+nodeName+" serverName="+serverName+" ..."
		appMgrID = AdminControl.queryNames("type=ApplicationManager,node="+nodeName+",process="+serverName+",*" )
		length = len(appMgrID)
		self.LOGGER.debug("StopApplication: appMgrID.length="+str(length)+" appMgrID="+appMgrID)
		if (length >= 1):
			print "stopApplication: stopping "+appName+"  ..."
			stopped = ""
			try:
				stopped = AdminControl.invoke(appMgrID, "stopApplication", '"' + appName + '"' )
			except:
				print "stopApplication: Exception trying to stop "+appName+" on node "+nodeName+" for server "+serverName
				print " Caused by: " + str(sys.exc_info()[1])
			#endTry 
			if (len(stopped) > 0):
				print stopped
			#endIf
		else:
			print "The application manager is not accessible.  The application " + appName + " cannot be stopped.";
			print ("This error usually indicates that the server '" + serverName 
				+ "' is not running and therefore the application is already stopped.");
		#endElse 
		self.LOGGER.traceExit()
	#endDef 
	
	def checkApplicationRunning (self, nodeName, serverName, appName ):
	
		self.LOGGER.traceEnter([nodeName, serverName, appName])
		print "checkApplicationRunning: "+nodeName+" "+serverName+" "+appName
		appID = ""
		try:
			_excp_ = 0
			appID = AdminControl.completeObjectName("type=Application,node="+nodeName+",Server="+serverName+",name="+appName+",*" )
			_excp_ = 0 #reset (in case of nested exceptions)
		except:
			_type_, _value_, _tbck_ = sys.exc_info()
			_excp_ = 1
		#endTry 
		temp = _excp_
		if (temp > 0):
			print "checkApplicationRunning: Exception trying to getID for "+appName+" "+nodeName+" "+serverName
		#endIf 
		length = len(appID)
		self.LOGGER.debug("checkApplicationRunning: appID.length="+str(length))

		retries = 0
		while (( retries < 20 )  and ( length == 0 ) ):
			retries = retries+1
			print "checkApplicationRunning: not yet started, "+appName+" "+nodeName+" "+serverName+", retries="+str(retries)+" ..."
			try:
				_excp_ = 0
				SystemUtils.sleepDelay(10 )
				appID = AdminControl.completeObjectName("type=Application,node="+nodeName+",Server="+serverName+",name="+appName+",*" )
				if (len(appID) == 0):
					appExists = self.checkIfAppExists(appName )
					if (not appExists):
						print "checkApplicationRunning: "+appName+" is NOT INSTALLED."
						self.LOGGER.traceExit()
						return 
					#endIf 
				#endIf 
				_excp_ = 0 #reset (in case of nested exceptions)
			except:
				_type_, _value_, _tbck_ = sys.exc_info()
				_excp_ = 1
			#endTry 
			temp = _excp_
			if (temp > 0):
				print "checkApplicationRunning: Exception trying to getID for "+appName+" "+nodeName+" "+serverName
			#endIf 
			length = len(appID)
			self.LOGGER.debug("checkApplicationRunning: temp="+str(temp)+" appID="+str(appID)+" ...")
		#endWhile 
		if (retries > 0):
			print "checkApplicationRunning: "+nodeName+" "+serverName+" "+appName+" had slow start, status retries="+str(retries)
		#endIf 

		if (length > 0):
			print "checkApplicationRunning: "+appName+" is STARTED."
		else:
			print "checkApplicationRunning: "+appName+" "+nodeName+" "+serverName+" DID NOT START."
		#endElse 
		self.LOGGER.traceExit()
	#endDef 

	"""
		Starts the application on each server 
		
		@param appNames: the application name, or a comma delimited list of applications
		@param nodeName: the name of the node which contains the server
		@param serverName: the name of the server that the application will be started/stopped on
		@param startOrStop: "start" or "stop"
	"""	
	def startStopApplication(self, appNames, nodeName, serverName, startOrStop):
		self.LOGGER.traceEnter([appNames, nodeName, serverName])
		
		for appName in appNames.split(","):
			if (startOrStop == "start"):
				self.startApplication(appName, nodeName, serverName )
			else:
				appExists = self.checkIfAppExists(appName )
				if (appExists):
					self.stopApplication(appName, nodeName, serverName )
				#endIf 
			#endIf
		#endFor
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Starts the application on each cluster member of the supplied cluster.
		
		@param appName: the application name, or a comma delimited list of applications
		@param clusterName: the name of the cluster that this application is mapped to
		@param startOrStop: "start" or "stop"
	"""
	def startStopAppOnCluster(self, appName, clusterName, startOrStop):
		self.LOGGER.traceEnter([clusterName])
		cellname = AdminControl.getCell()
		cluster = AdminConfig.getid("/ServerCluster:"+clusterName+"/" )
		if (len(cluster) > 0):
			memberlist = AdminConfig.showAttribute(cluster, "members" )
			members = memberlist[1:len(memberlist)-1].split()
			for member in members:
				nodeName = AdminConfig.showAttribute(member, "nodeName" )
				serverName = AdminConfig.showAttribute(member, "memberName" )
				## check to see if this server is an application server
				scope = '/Cell:' + cellname + '/Node:' + nodeName + '/Server:' + serverName + '/'
				serverId = AdminConfig.getid(scope)
				type = AdminConfig.showAttribute(serverId, 'serverType')
				if (type == 'APPLICATION_SERVER'):
					self.startStopApplication( appName, nodeName, serverName, startOrStop)
				else:
					self.LOGGER.info("The cluster member is not an application server.  Skipping application " 
							+ startOrStop + " for this member: " + serverName)
				#endIf
			#endFor 
		else:
			print "Error: Unable to find cluster " + clusterName + " in runtime configuration"
			print "	   Unable to start or stop application"
			raise "Unable to find cluster " + clusterName + " in runtime configuration"
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Determines which servers/clusters this application is mapped to 
		and starts/stops the app on those mapped servers/clusters.
	"""
	def startStopAppOnModuleMappedServers(self, appName, startOrStop ):
		self.LOGGER.traceEnter([appName])
		cellname = AdminControl.getCell()
		appExists = self.checkIfAppExists(appName )
		if (appExists):
			alreadyStarted = {}
			for moduleServer in AdminApp.listModules(appName, '-server').split(newline):
				# string returned is in the form 
				# PlantsByWebSphere#PlantsGallery.war+WEB-INF/web.xml#WebSphere:cell=cell,node=dmgr,server=dmgr
				# PlantsByWebSphere#PlantsByWebSphere.war+WEB-INF/web.xml#WebSphere:cell=cell,cluster=cluster
				if (len(moduleServer) > 0):
					segments = moduleServer.split('#')
					if (len(segments) >= 3):
						mappedServers = segments[2]
						## string may be WebSphere:cell=cell,cluster=cluster+WebSphere:cell=cell,cluster=cluster
						for mappedServer in mappedServers.split('+'):
							self.LOGGER.debug("Processing server mapping: " + mappedServer)
							if (mappedServer.find(':') > 0):
								scopeDict = {}
								scope = mappedServer.split(':')[1]
								for scopeArg in scope.split(','):
									name,value = scopeArg.split('=')
									scopeDict[name] = value
								#endFor
								if (scopeDict.has_key("cluster")):
									clusterName = scopeDict["cluster"]
									if (not alreadyStarted.has_key(clusterName)):
										self.startStopAppOnCluster(appName, clusterName, startOrStop)
										alreadyStarted[clusterName] = 1
									#endIf
								elif (scopeDict.has_key("node") and scopeDict.has_key("server")):
									nodeName = scopeDict["node"]
									serverName = scopeDict["server"]
									if (not alreadyStarted.has_key(nodeName + serverName)):
										scope = '/Cell:' + cellname + '/Node:' + nodeName + '/Server:' + serverName + '/'
										serverId = AdminConfig.getid(scope)
										type = AdminConfig.showAttribute(serverId, 'serverType')
										if (type == 'APPLICATION_SERVER'):
											self.startStopApplication(appName, nodeName, serverName, startOrStop)
										else:
											self.LOGGER.info("Server is not an application server. Skipping application " +
													startOrStop + " for server: " + serverName)
										#endIf
										alreadyStarted[nodeName + serverName] = 1
									#endIf
								else:
									print "Error: unable to determine cluster, node or server on which the application is deployed."
								#endIf
							else:
								print "Error: unable to determine cluster, node or server on which the application is deployed."
							#endIf
						#endFor	
					else:
						print "Error: unable to determine cluster, node or server on which the application is deployed."
					#endIf
				#endIf
			#endFor
		#endIf - App Exists
		self.LOGGER.traceExit()
	#endDef	
#endClass


## define global singletons
WasDeploy = _WasDeploy()	

