"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: XmlProperty.py
	
	Defines XmlProperty and XmlPropertyNode.
"""


from    java.io import StringReader
from    org.python.modules import re
import  org.xml.sax as sax
import  org.apache.xerces.parsers.DOMParser as domparser
import  org.w3c.dom.traversal.NodeFilter as NodeFilter
import  org.w3c.dom.traversal as traversal
import  org.w3c.dom.Node as Node

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from com.ibm.rational.rafw.wsadmin.util import WsadminStringUtils

from java.util import HashMap

import Globals

############## CLASS :: XmlProperty ###########################
# Use this class to handle xml property files inside          #
# RAFW                                                        #
#                                                             #
# Example: argv[0] = <somefile> argv[1] = some xml-tag        #
#	xmlprop  = XmlProperty( sys.argv[0] )                     #
#	nodelist = xmlprop.getFilteredNodeList( sys.argv[1] )     #
#	nodeHash = xmlprop.xmlReadNodeLevel( nodelist )           #
###############################################################
class XmlProperty:

	"""
	This constructor expects to find an xml file to read in.
	
	The following filters are always applied when identifying attribute to write to WAS
		SkipNonWASAttributes -- used to skip RAFWType and WasKey attributes from being written to WAS
		SkipEmptyAttributes  -- used to skip the attribute named 'name' when there is no value assigned to it. 	
	"""
	def __init__(self, file):
		
		self.LOGGER = _Logger("XmlProperty", MessageManager.RB_WEBSPHERE_WAS)
		self.LOGGER.traceEnter(file)
		parser = domparser()
		document = open(file).read()
		parser.reset()
		documentIS = sax.InputSource(StringReader(document))
		parser.parse(documentIS)
		self.domDoc = parser.getDocument()
		
		self.skipAttributeOnWriteFilters = []
		self.skipAttributeOnWriteFilters.append(SkipNonWASAttributes())
		self.skipAttributeOnWriteFilters.append(SkipEmptyAttributes('name'))
		self.LOGGER.traceExit()
	#endDef

	###########################################################
	# returns a NodeList based on the tag passed in
	def getFilteredNodeList(self, tag):
		return self.domDoc.getElementsByTagName(tag)
	#endDef

	#
	# apply the attributes to the passed in attr hash
	def setHashAttributes(self, node, attrHash):
		attributes = node.getAttributes()
		for i in range(attributes.getLength()):
			attrHash[attributes.item(i).getNodeName()] = attributes.item(i).getNodeValue()
		#endFor
	#endDef

	##
	## returns an array of XmlPropertyNodes which are children 
	## of the top level node and whose tag names match the tag passed in
	## Each returned object is a XmlPropertyNode
	def getFilteredNodeArray(self, tag):
		self.LOGGER.traceEnter(tag)
		nodeArray = []
		outerNode = self.domDoc.getDocumentElement()
		if (outerNode != None):
			self.LOGGER.debug("Processing outer node:" + outerNode.getNodeName())
			nodeList = outerNode.getChildNodes()
			self.LOGGER.debug("Found " + str(nodeList.getLength()) + " child nodes")
			for ix in range(nodeList.getLength()):
				xmlChild = nodeList.item(ix)
				self.LOGGER.debug("Found child node:" + xmlChild.getNodeName())
				if (xmlChild.getNodeName() == tag):
					nodeArray.append(XmlPropertyNode(xmlChild, self))
				#endIf
			#endFor
		#endIf
		self.LOGGER.traceExit(nodeArray)
		return nodeArray
	#endDef

	##
	## returns an array of XmlPropertyNodes which are children 
	## of the top level node
	## Each returned object is a XmlPropertyNode
	def getNodeArray(self):
		self.LOGGER.traceEnter()
		nodeArray = []
		outerNode = self.domDoc.getDocumentElement()
		if (outerNode != None):
			self.LOGGER.debug("Processing outer node:" + outerNode.getNodeName())
			nodeList = outerNode.getChildNodes()
			self.LOGGER.debug("Found " + str(nodeList.getLength()) + " child nodes")
			for ix in range(nodeList.getLength()):
				xmlChild = nodeList.item(ix)
				self.LOGGER.debug("Found child node:" + xmlChild.getNodeName())
				nodeArray.append(XmlPropertyNode(xmlChild, self))
			#endFor
		#endIf
		self.LOGGER.traceExit(nodeArray)
		return nodeArray
	#endDef

	#
	# traverse one node level and return what you got there as a hash
	# This function will call itself recursively to populate the dictionary
	def xmlReadNodeLevel(self, NodeList):
		thisNodeList = []
		ws = re.compile("\s")
		for ix in range(NodeList.getLength()):
			attrHash = {}

			thisNode = NodeList.item(ix)
			thisName = thisNode.getNodeName()
	
			# If the current node is a TEXT_NODE then apply it's value to the parent, but only do
			# so if it's not just a bunch of whitespace
			if (thisNode.getNodeType() == Node.TEXT_NODE):
				if (ws.match(thisNode.getNodeValue())):
					pass
				else:
					#attrHash[thisName] = { thisNode.getParentNode().getNodeName() : thisNode.getNodeValue() }
					attrHash[thisNode.getParentNode().getNodeName()] = thisNode.getNodeValue()
				#endIf
			#endIf

			# Grab all of the attributes defined for this level
			if (thisNode.hasAttributes()):
				attrHash[thisName] = {}
				self.setHashAttributes(thisNode, attrHash[thisName])
			#endIf
	
			# if this node has children then decide if we need to make recursive call to feed in next level
			if (thisNode.hasChildNodes()):
				if (thisNode.getChildNodes().getLength() > 1):
					## if thisNode doesn't have attributes, then the attrHash[thisName]
					## will not be initialized, and the attrHash[thisName]["children"] 
					## assignment will fail
					if (not attrHash.keys()):
						attrHash[thisName] = {}
					#endIf
					#attrHash[thisName] = self.xmlReadNodeLevel( thisNode.getChildNodes() )
					attrHash[thisName]["children"] = self.xmlReadNodeLevel(thisNode.getChildNodes())
				else:
					child = NodeList.item(ix).getFirstChild()
					if (child.getNodeType() == child.TEXT_NODE) and (child.getNodeValue() != ""):
						if (ws.search(child.getNodeValue())):
							# maybe some day I should make this work, but right now we don't need text for property files
							#attrHash[thisName] = child.getNodeValue()
							pass
						else:
							pass
						#endIf
					#endIf
				#endIf
			#endIf
			
			if (attrHash.keys()):
				thisNodeList.append(attrHash)
			#endIf
		#endFor
		return thisNodeList
	#endDef
	
	"""
		Returns an XmlPropertyNode that contains the configuration data being managed
		for the containerType
	"""
	def findRAFWContainerNode(self, containerType):
		markerNodes = self.getFilteredNodeArray(Globals.RAFW_XML_PREFIX + containerType)
		if (len(markerNodes) > 0):
			## using new file format
			return markerNodes[0]
		#endIf
		return XmlPropertyNode(self.domDoc.getDocumentElement(), self)
	#endDef

	"""
	Adds the following filter to the list of filters applied while building attribute 
	name/value pairs.  The attributes will be used when writing configuration in 
	WebSphere, either via AdminConfig or AdminTask
	
	These filters are not used while importing or comparing configuration.
	
	Filters on the XmlProperty object are global and are applied to all child XmlPropertyNode instances
	
	The following filters are always applied
	SkipNonWASAttributes -- used to skip RAFWType and WasKey attributes from being written to WAS
	SkipEmptyAttributes  -- used to skip the attribute named 'name' when there is no value assigned to it. 	
	"""	
	def addSkipAttributeOnWriteFilter(self, skipAttrOnWriteFilter):
		if (skipAttrOnWriteFilter is not None):
			self.skipAttributeOnWriteFilters.append(skipAttrOnWriteFilter)
		#endIf
	#endDef

	"""
	Applies the filters to identify which attributes  will be written to WAS.
	
	Filters on the XmlProperty object are global and are applied to all child XmlPropertyNode instances
	
	If any filter in the skipAttributeOnWriteFilters returns true(1), then this method
	will return true(1).  If no filters apply, returns false (0)
	"""	
	def applySkipAttributeOnWriteFilters(self, nodeName, key, value):
		for filter in self.skipAttributeOnWriteFilters:
			if (filter.skipAttribute(nodeName, key, value)):
				return 1
			#endIf
		#endFor
		return 0
	#endDef

	"""
	Clear all previously applied non-default filters to identify 
	which attributes will be written to WAS.
	
	Filters on the XmlProperty object are global and are applied to all child XmlPropertyNode instances.  
	You must also clear any filters set on the specific XmlPropertyNode instance
	
	All default filters will be retained as is.
	"""	
	def clearSkipAttributeOnWriteFilters(self):
		self.skipAttributeOnWriteFilters = []
		self.skipAttributeOnWriteFilters.append(SkipNonWASAttributes())
		self.skipAttributeOnWriteFilters.append(SkipEmptyAttributes('name'))
	#endDef
#endClass

############## CLASS :: XmlPropertyNode ###################
# Use this class to handle xml property nodes inside      #
# RAFW                                                    #
#                                                         #
###########################################################
class XmlPropertyNode:

	"""
	This constructor expects to wrap an xml node.
	
	parameters:
	xmlNodeToWrap is the XML node from the dom tree.
	xmlProperty is the XmlProperty instance used when creating this instance.
	parent is the XmlPropertyNode who contains this instance as a child.

	This constructor is never used outside of this .py module.  All methods in
	this class and the XmlProperty class create XmlPropertyNode instances to 
	be used by clients
	"""
	def __init__(self, xmlNodeToWrap, xmlProperty, parent=None):
		self.LOGGER = _Logger("XmlPropertyNode", MessageManager.RB_WEBSPHERE_WAS)
		self.xmlNode = xmlNodeToWrap
		self.configId = None
		self.filteredChildren = {}
		self.childrenArray = None
		self.xmlProperty = xmlProperty
		self.parent = parent
		self.skipAttributeOnWriteFilters = []
	#endDef

	"""
	Adds the following filter to the list of filters applied while building attribute 
	name/value pairs.  The attributes will be used when writing configuration in 
	WebSphere, either via AdminConfig or AdminTask
	
	These filters are not used while importing or comparing configuration.
	
	Filters on the XmlPropertyNode object are specific to data returned by this instance.
	
	Global filters on the XmlProperty parent are also applied
	
	"""	
	def addSkipAttributeOnWriteFilter(self, skipAttrOnWriteFilter):
		if (skipAttrOnWriteFilter is not None):
			self.skipAttributeOnWriteFilters.append(skipAttrOnWriteFilter)
		#endIf
	#endDef

	"""
	Applies the filters to identify which attributes  will be written to WAS.
	
	Filters on the XmlPropertyNode object are specific to data returned by this instance.
	Global filters on the XmlProperty parent are also considered
	
	If any filter in the skipAttributeOnWriteFilters returns true(1), then this method
	will return true(1).  If no filters apply, returns false (0)
	"""	
	def _applySkipAttributeOnWriteFilters(self, nodeName, key, value):
		if (self.xmlProperty.applySkipAttributeOnWriteFilters(nodeName, key, value)):
			return 1
		else:
			for filter in self.skipAttributeOnWriteFilters:
				if (filter.skipAttribute(nodeName, key, value)):
					return 1
				#endIf
			#endFor
		#endIf
		return 0
	#endDef

	"""
	Clear all previously applied non-default filters to identify 
	which attributes will be written to WAS.
	
	Filters on the XmlPropertyNode object are specific to data returned by this instance.
	Global filters on the XmlProperty parent are not impacted by this method.
	"""	
	def clearSkipAttributeOnWriteFilters(self):
		self.skipAttributeOnWriteFilters = []
	#endDef
	
	def getParent(self):
		"""
		Returns the parent XmlPropertyNode of this object, or None if no parent was set 
		when this instance was created.
		"""
		return self.parent
	#endDef
		
	###########################################################
	## Returns the xml node that this object wraps, 
	## this is the underlying java XML Node
	###########################################################
	def getXmlNode(self):
		return self.xmlNode
	#endDef

	###########################################################
	## If the name of this node is X.Y then return Y without the dot
	##
	## Example: if "ApplicationServer.Classloader" is the name of the node
	##          then return "Classloader"
	###########################################################
	def getNodeNameFixed(self):
		nodeName = self.xmlNode.getNodeName()
		lastIndexOfDot = nodeName.rfind('.') + 1
		if (lastIndexOfDot > 0):
			end = len(nodeName)
			nodeName = nodeName[lastIndexOfDot:end]
		#endIf
		return nodeName
	#endDef
	
	###########################################################
	##
	## returns the value for the supplied attribute
	###########################################################
	def getAttrValue(self, attr, default=None):
		namedNode = self.xmlNode.getAttributes().getNamedItem(attr)
		if (namedNode is None):
			return default
		else:
			return namedNode.getNodeValue()
	#endDef
	
	"""
		returns the value for the supplied attributes as a dictionary
		
		If an attribute does not have a value then it is excluded from 
		the returned dictionary
	
	"""
	def getAttrValues(self, attrs):
		xmlAttrs = self.xmlNode.getAttributes()
		attrValues = {}
		for attr in attrs:
			namedNode = xmlAttrs.getNamedItem(attr)
			if (namedNode is not None):
				attrValues[attr] = namedNode.getNodeValue()
		#endFor
		return attrValues
	#endDef
	
	###########################################################
	##
	## returns the value for the supplied attribute
	###########################################################
	def setAttrValue(self, attr, value):
		namedNode = self.xmlNode.getAttributes().getNamedItem(attr)
		if (namedNode is None):
			namedNode = self.xmlNode.getOwnerDocument().createAttribute(attr)
			self.xmlNode.getAttributes().setNamedItem(namedNode)
		#endIf
		namedNode.setNodeValue(value)
	#endDef

	###########################################################
	##
	## returns 1 if the node contains the supplied attribute,
	## else 0
	##
	###########################################################
	def hasAttr(self, attr):
		return self.hasAllAttrs([attr])
	#endDef
	
	###########################################################
	##
	## returns 1 if the node contains all of the supplied attributes,
	## else 0
	##
	###########################################################
	def hasAllAttrs(self, attrs):
		xmlAttrs = self.xmlNode.getAttributes()
		for attr in attrs:
			namedNode = xmlAttrs.getNamedItem(attr)
			if (namedNode is None):
				return 0
			#endIf
		#endFor
		return 1
	#endDef
			
	###########################################################
	##
	## returns a NodeList based on the tag passed in which are 
	## children of the supplied parent, converted to an array
	## wrapped by XmlPropertyNode
	###########################################################
	def getFilteredChildrenArray(self, tag):
		if (self.filteredChildren.has_key(tag) and self.filteredChildren[tag] is not None):
			return self.filteredChildren[tag]
		#endIf
		nodeList = self.xmlNode.getChildNodes()
		nodeArray = []
		for ix in range(nodeList.getLength()):
			xmlChild = nodeList.item(ix)
			if (xmlChild.getNodeName() == tag):
				nodeArray.append(XmlPropertyNode(xmlChild, self.xmlProperty, self))
			#endIf
		#endFor
		self.filteredChildren[tag] = nodeArray
		return nodeArray
	#endDef

	"""
	An alias for getFilteredChildrenArray to allow XmlProperty and XmlPropertyNodes 
	to share a similar interface.
	"""
	def getFilteredNodeArray(self, tag):
		return self.getFilteredChildrenArray(tag)
	#endDef

	###########################################################
	## Returns all children of the node, except TEXT_NODE and COMMENT_NODEs
	def getChildrenArray(self):
		if (self.childrenArray is not None):
			return self.childrenArray
		#endIf
		nodeList = self.xmlNode.getChildNodes()
		nodeArray = []
		for ix in range(nodeList.getLength()):
			childNode = nodeList.item(ix)
			childNodeType = childNode.getNodeType()
			if ((childNodeType != Node.TEXT_NODE) and (childNodeType != Node.COMMENT_NODE)):
				nodeArray.append(XmlPropertyNode(childNode, self.xmlProperty, self))
			#endIf
		#endFor
		self.childrenArray = nodeArray
		return nodeArray
	#endDef

	"""
		Builds a 2 dimensional array of attributes for use
		in the AdminConfig.modify and AdminConfig.create methods
		
		All filters in the SkipAttributeOnWriteFilters list associated with the XmlProperty
		object will be applied.  If any return true, the attribute is excluded from the array.
		
		excludes attributes: Globals.WAS_KEY_ATTR_NAME and Globals.RAFW_TYPE
	"""
	def buildNodeAttrs(self, excludedNames=[]):
		self.LOGGER.traceEnter([excludedNames])
		attributes = self.xmlNode.getAttributes()
		xmlNodeName = self.xmlNode.getNodeName()
		attrs = []
		if (attributes != None):
			for i in range(attributes.getLength()):
				thisAttr = attributes.item(i)
				
				thisNodeName = thisAttr.getNodeName()
				thisNodeValue = thisAttr.getNodeValue()
				# apply filters on the dictionary
				if (self._applySkipAttributeOnWriteFilters(xmlNodeName, thisNodeName, thisNodeValue)):
					continue
				#endIf
				## If this node name is not in the list of excluded names
				if (thisNodeName not in excludedNames):
					attrs.append([thisNodeName, thisNodeValue])
				#endIf
			#endFor	
		#endIf
		self.LOGGER.traceExit([attrs])
		return attrs

	#endDef

	""" 
		Converts the attributes of this xml node to a 
		string of the form [[name "value"][name2 "value2" ]]
		
		If the attribute name is supplied and has a blank 
		value (empty string), then the key/value pair are not 
		included in the output.
		
		All filters in the SkipAttributeOnWriteFilters list associated with the XmlProperty
		object will be applied.  If any return true, the attribute is excluded from the array.
	
		Including:	
		The attribute with the name	Globals.WAS_KEY_ATTR_NAME is ignored.
		The attribute with the name	Globals.RAFW_TYPE is ignored.
		
		All attributes with a name in excludedNames are ignored
		
		The returned object is a string.
	"""
	def buildNodeAttrsAsJyString(self, excludedNames=[]):
		self.LOGGER.traceEnter(excludedNames)
		attrs = self.buildNodeAttrsDict()
		xmlNodeName = self.xmlNode.getNodeName()
		attrStr = "["
		for key in attrs.keys():
			# apply filters on the dictionary
			if (self._applySkipAttributeOnWriteFilters(xmlNodeName, key, attrs[key])):
				continue
			#endIf
			if (key not in excludedNames):
				value = attrs[key]
				attrStr = attrStr + self._wrapValue(key, value)
			#endIf
		#endFor
		attrStr = attrStr + "]"
		self.LOGGER.traceExit(attrStr)
		return attrStr
	#endDef	
		
	"""
		Builds a 1 dimensional array of attributes for use
		in the AdminTask methods.
	
		All filters in the SkipAttributeOnWriteFilters list associated with the XmlProperty
		object will be applied.  If any return true, the attribute is excluded from the array.
	"""	
	def buildNodeAttrsList(self):
		self.LOGGER.traceEnter()

		attributes = self.xmlNode.getAttributes()
		xmlNodeName = self.xmlNode.getNodeName()
		attrs = []
		for i in range(attributes.getLength()):
			thisAttr = attributes.item(i)
			thisNodeName = thisAttr.getNodeName()
			thisNodeValue = thisAttr.getNodeValue()
			# apply filters on the dictionary
			if (self._applySkipAttributeOnWriteFilters(xmlNodeName, thisNodeName, thisNodeValue)):
				continue
			#endIf
			attrs.append("-" + thisNodeName)
			attrs.append(thisNodeValue)
		#endFor	
		self.LOGGER.traceExit(attrs)
		return attrs

	#endDef
	
	"""
		Builds a dict with the attributes of this node
		Includes the Globals.WAS_KEY_ATTR_NAME since this dict is used for comparison
	
		No filters in the SkipAttributeOnWriteFilters list associated with the XmlProperty
		object will be applied since this dict is used for comparison and not 
		for configuration modification
	"""
	def buildNodeAttrsDict(self):

		attributes = self.xmlNode.getAttributes()
		attrs = {}
		for i in range(attributes.getLength()):
			thisAttr = attributes.item(i)
			thisNodeName = thisAttr.getNodeName()
			attrs[thisNodeName] = thisAttr.getNodeValue()
		#endFor	
		return attrs

	#endDef
	
	"""
	Same as buildNodeAttrsDict but returns a Map
	"""
	def buildNodeAttrsMap(self):

		attributes = self.xmlNode.getAttributes()
		attrs = HashMap()
		for i in range(attributes.getLength()):
			thisAttr = attributes.item(i)
			thisNodeName = thisAttr.getNodeName()
			attrs.put(thisNodeName, thisAttr.getNodeValue())
		#endFor	
		return attrs
	#endDef
	
	"""
		Get the WAS config object ID if one has been set for this node.
		
		Returns None if a WAS config object ID has not been set on this instance.
	"""
	def getConfigId(self):
		self.LOGGER.traceEnter()
		self.LOGGER.traceExit(self.configId)
		return self.configId
	#endDef
	
	"""
		Used to set the WAS config object ID after the attributes in this 
		object have been created or modified in a WAS runtime configuration
	"""
	def setConfigId(self, objectId):
		self.LOGGER.traceEnter([objectId])
		self.configId = objectId
		self.LOGGER.traceExit()
	#endDef

	"""
		Override default behavior to tell us a little more about this object.
	"""
	def __str__(self):
		return self.xmlNode.getNodeName() + str(self.buildNodeAttrsDict())
	#endDef
	
	def _wrapValue(self, key, value):
		wrappedValue = ""
		# AdminConfig.create will not allow us to escape quotes
		# with \" or \' so we have to do this logic
		containsSQuote = value.find("'") != -1
		containsDQuote = value.find("\"") != -1
		if(containsSQuote and containsDQuote):
			nodeName = self.xmlNode.getNodeName()
			self.LOGGER.log("CRWWA3200I", [nodeName, key, value])
			# if we can normalize, lets do it
			if (WsadminStringUtils.quotesMatch(value)):
				self.LOGGER.log("CRWWA3201I", [nodeName, key, value])
				## normalize quotes will replace ' for ", 
				## so the string will only contain double quotes
				value = WsadminStringUtils.normalizeQuotes(value);
				# wrap double-quotes in single-quotes
				wrappedValue = '[' + key + " '" + value + "']"
			else:	
				self.LOGGER.log("CRWWA3202E", [nodeName, key, value])
				## else raise the error and ask the user to change the data file
				#raise InvalidXmlConfigException(
				#		MessageManager.getExceptionMessage(
				#				MessageManager.RB_WEBSPHERE_WAS, "CRWWA3202E", [nodeName, key, value]))
		elif(containsSQuote):
			# wrap single-quotes in double quotes
			wrappedValue = '[' + key + ' "' + value + '"]'
		else:
			# wrap double-quotes in single-quotes
			wrappedValue = '[' + key + " '" + value + "']"
		return wrappedValue
	#endDef
#endClass

"""
	Thrown when XML configuration is not valid and processing cannot continue.
"""	
class InvalidXmlConfigException:
	
	def __init__(self, message):
		self.message = message
	#endDef
	
	def __str__(self):
		return str(self.message)
	#endDef
	
#endClass

"""
	Implement this interface to define filtering rules around which attributes are 
	returned from the XML document to be written to WAS
"""
class ISkipAttributeOnWriteFilter:
	"""
		If skipAttribute returns true(1) then the attribute will not be included in
		the datastructure that is used to write configuration to WAS
		
		nodeName - The name of the XML node, this corresponds to the WAS Config Type
		key      - The name of the attribute
		value    - The value of the attribute 	
	"""
	def skipAttribute(self, nodeName, key, value):
		return 0
	#endDef
#endClass

"""
	Skips RAF specific attributes including WasKey and RAFWType
"""
class SkipNonWASAttributes(ISkipAttributeOnWriteFilter):
	def skipAttribute(self, nodeName, key, value):
		if (key == Globals.WAS_KEY_ATTR_NAME):
			return 1
		if (key == Globals.RAFW_TYPE):
			return 1
		#endIf
		return 0
	#endDef
#endClass

"""
	For any reason whatsoever if an attribute is to be skipped, it will be skipped
"""
class SkipNamedAttribute(ISkipAttributeOnWriteFilter):
	"""
		Construct this instance.
	
		keyName is the name of the attribute that should be skipped
	"""
	def __init__(self, keyName):
		self.keyName = keyName
	#endDef

	def skipAttribute(self, nodeName, key, value):
		return (self.keyName == key)
	#endDef
#endClass

"""
	Skips attributes whose value is "" 
"""
class SkipEmptyAttributes(ISkipAttributeOnWriteFilter):
	"""
		Construct this instance.
	
		keyName is the name of the attribute whose value must not be ""
	"""
	def __init__(self, keyName):
		self.keyName = keyName
	#endDef

	"""
		Skip the attribute if the key matches the one supplied in the constructor 
		and the value is ""
	"""	
	def skipAttribute(self, nodeName, key, value):
		if (key == self.keyName and value == ""):
			return 1
		#endIf
		return 0
	#endDef
#endClass

"""
	Skips attributes whose value is "" and which are contained by
	the specified xml node name.
"""
class SkipEmptyAttributesForNode(ISkipAttributeOnWriteFilter):
	"""
		Construct this instance.

		nodeName is the name of the node which must contain the attribute	
		keyName is the name of the attribute whose value must not be ""
	"""
	def __init__(self, nodeName, keyName):
		self.nodeName = nodeName
		self.keyName = keyName
	#endDef
	
	"""
		Skip the attribute if the key matches the one supplied in the constructor,
		the value is "" and the name of the node containing the attribute matches
		the one supplied to the constructor
	"""	
	def skipAttribute(self, nodeName, key, value):
		if (nodeName == self.nodeName and key == self.keyName and value == ""):
			return 1
		#endIf
		return 0
	#endDef
#endClass
