"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: CellDefinitionReader.py
	
	Reads properties from an existing cell which are used to generate an 
	Environment Cell Definition from an existing WebSphere environment.
	
"""

"""
In versions 6.0 and 6.1, these objects are not globally
available in modules.  This allows us to access these
objects directly in modules such as ConfigReader and ConfigWriter

To use these objects in a module, add an import declaration near
the top of the module.  For example:

  import AdminConfig

"""
import sys

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

try:
	if (AdminConfig is None or AdminTask is None or AdminApp is None or AdminControl is None):
		# raise a name error in case it hasn't already happened when we tried to test the above object states
		raise NameError
	else:
		sys.modules['AdminConfig'] = AdminConfig  
		sys.modules['AdminControl'] = AdminControl  
		sys.modules['AdminApp'] = AdminApp  
		sys.modules['AdminTask'] = AdminTask  
	#endIf
except NameError:
	_Logger("CellDefinitionReader", MessageManager.RB_WEBSPHERE_WAS).log("CRWWA0146E")
	raise MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS, "CRWWA0146E", None)
#endTry

from java.io import File
from SystemUtils import SystemUtils
from existingcell.WasCellProcessor import WasCellProcessor
from existingcell.IhsCellProcessor import IhsCellProcessor
from com.ibm.rational.rafw.wsadmin.util import Globals, WsadminPropertiesUtil
from com.ibm.rational.rafw.wsadmin.util import PropertyFileWriter
from JavaConversionHelper import JavaConversionHelper
from existingcell.ProcessorFactory import ProcessorFactory
from AdminHelper import AdminHelper

from existingcell.Constants import NEWLINE

import AdminConfig
import AdminTask

"""
	Reads properties from an existing cell which are used to generate an 
	Environment Cell Definition from an existing WebSphere environment.
"""
class CellDefinitionReader:
	
	ADMIN_MANAGED_NODE_PREFIX = "NODE"
	ADMIN_AGENT_NODE_PREFIX = "ADMIN_AGENT_"
	
	def __init__(self):
		self.LOGGER = _Logger("CellDefinitionReader", MessageManager.RB_WEBSPHERE_WAS)
		self.adminAgentManagedNodesDict = {}
		self.environmentDict = {}
		self.processors = []
		self.ihsNodeNum = 0
		self.useSoapProps = 'false'
		self.username = ''
		self.password = ''
	#endDef

	def addProcessor(self, processor):
		self.processors.append(processor)
	#endDef
	
	"""
		Reads the cell configuration values.  Call this method to populate the 
		environment dictionary with properties which represent the cell configuration.
	"""
	def processRegularCell(self, cell):
		for processor in self.processors:
			processor.cellDiscovered(cell)
		#endFor		
		
		for node in AdminHelper.listNodes():
			for processor in self.processors:
				processor.nodeDiscovered(node)
			#endFor		
			for server in AdminConfig.list('Server', node).split(NEWLINE):
				## server will be "" if there are no servers on this node
				if (len (server) > 0):
					for processor in self.processors:
						processor.serverDiscovered(node, server)
					#endFor
				#endIf
			#endFor
		#endFor	

		## read cluster data
		for cluster in AdminConfig.list( 'ServerCluster' ).split( NEWLINE ):
			if (len(cluster) > 0):
				for processor in self.processors:
					processor.clusterDiscovered(cluster)
				#endFor		
			#endIf
		#endFor
	#endDef

	def processAdminAgentCell(self, cell):
		
		for processor in self.processors:
			processor.setIsAdminAgentCell("true")
			processor.cellDiscovered(cell);
			
		for node in AdminHelper.listNodes():
			for processor in self.processors:
				processor.nodeDiscovered(node)
			#endFor		
			for server in AdminConfig.list('Server', node).split(NEWLINE):
				## server will be "" if there are no servers on this node
				if (len (server) > 0):
					for processor in self.processors:
						processor.serverDiscovered(node, server)
					#endFor
				#endIf
			#endFor
		#endFor	
		
		for processor in self.processors:
			if (isinstance(processor, WasCellProcessor)):
				processor.managedNodeNum = 0
			#endIf
		#endFor
		
		nodePropPattern = java.util.regex.Pattern.compile('\\s*(profile\\.\\S+)+')
		nodePortPattern = java.util.regex.Pattern.compile('port\\s*(\\d+)')
		nodeHostPattern = java.util.regex.Pattern.compile('host\\s*(.+?)\\s*]')
		managedNodes = AdminConfig.list('ManagedNode').split(NEWLINE)
		if (managedNodes == ""):
			return
		for nodeId in managedNodes:
			nodeName = AdminConfig.showAttribute(nodeId, 'name')
			self.adminAgentManagedNodesDict[nodeName] = []
			properties = AdminConfig.showAttribute(nodeId, 'properties').strip().split('[')[1].split(']')[0]
			nodePropMatcher = nodePropPattern.matcher(java.lang.String(properties))
			nodeProps = {}
			
			while (nodePropMatcher.find() > 0):
				propId = nodePropMatcher.group(1)
				propKey = propId.split('(')[0]
				propVal = AdminConfig.showAttribute('(' + propId.split('(')[1], 'value')
				nodeProps[propKey] = propVal
			#endWhile
			nodeConnProps = AdminTask.getManagedNodeConnectorProperties('[ -connType SOAP -managedNodeName ' + nodeName + ' ]')
			
			nodePortMatcher = nodePortPattern.matcher(java.lang.String(nodeConnProps))
			if (nodePortMatcher.find() > 0):
				nodePort = nodePortMatcher.group(1)
				self.adminAgentManagedNodesDict[nodeName].append(['Port', nodePort])
			#endIf
			
			nodeHostMatcher = nodeHostPattern.matcher(java.lang.String(nodeConnProps))
			if (nodeHostMatcher.find() > 0):
				nodeHost = nodeHostMatcher.group(1)
				self.adminAgentManagedNodesDict[nodeName].append(['Host', nodeHost])
			#endIf
			
			nodeCellName = nodeProps['profile.registry.cell.name']
			self.adminAgentManagedNodesDict[nodeName].append(['ProfileCell', nodeCellName])
			nodeProfileRoot = nodeProps['profile.registry.profile.root'].replace('\\', '/')
			self.adminAgentManagedNodesDict[nodeName].append(['ProfileRoot', nodeProfileRoot])
			nodeWASHome = nodeProps['profile.registry.was.home'].replace('\\', '/')
			self.adminAgentManagedNodesDict[nodeName].append(['ProfileWASHome', nodeWASHome])

			version = nodeProps['profile.registry.profile.version']
			major,minor,point = version.split('.', 2)
			
			for processor in self.processors:
				processor.nodeDiscovered(nodeId)
				processor.discoveredNodes[nodeId].wasVersion = major + minor
				processor.discoveredNodes[nodeId].fullVersion = version
			#endFor		
			
			#set was version for this node
			servers = self.runManagedNodeWsadminCmd(nodeProfileRoot, nodeHost, nodePort, "print AdminConfig.list('Server')", "\\(.+?\\)")
			if (len(servers) > 0):
				serverType = ''
				serverName = ''
				serverTypePatternString = "serverType\\s+([A-Z]+_[A-Z]+)"
				serverNamePatternString = "server\\s*(.+?)\\]"
				for serverId in servers:
					serverTypeArr = self.runManagedNodeWsadminCmd(nodeProfileRoot, nodeHost, nodePort, "print AdminConfig.showAttribute('" + serverId + "', 'serverType')", "^[A-Z]+_[A-Z]+$")
					if (len(serverTypeArr) == 0): #AdminConfig returned blank serverType, lets try AdminTask
						serverTypeArr = self.runManagedNodeWsadminCmd(nodeProfileRoot, nodeHost, nodePort, "print AdminTask.showServerInfo('" + serverId + "')", serverTypePatternString)
						if (len(serverTypeArr) == 0):
							raise NameError
						#endIf
						serverTypePattern = java.util.regex.Pattern.compile(serverTypePatternString)
						serverTypeMatcher = serverTypePattern.matcher(java.lang.String(serverTypeArr[0]))
						if (serverTypeMatcher.find()):
							serverType = serverTypeMatcher.group(1)
						#endIf
					else:
						serverType = serverTypeArr[0]
					#endIf
					
					serverNameArr = self.runManagedNodeWsadminCmd(nodeProfileRoot, nodeHost, nodePort, "print AdminConfig.showAttribute('" + serverId + "', 'name')", ".*")
					if (len(serverNameArr) == 0): #AdminConfig returned blank serverName, lets try AdminTask
						serverNameArr = self.runManagedNodeWsadminCmd(nodeProfileRoot, nodeHost, nodePort, "print AdminTask.showServerInfo('" + serverId + "')", serverNamePatternString)
						if (len(serverNameArr) == 0):
							raise NameError
						#endIf
						serverNamePattern = java.util.regex.Pattern.compile(serverNamePatternString)
						serverNameMatcher = serverNamePattern.matcher(java.lang.String(serverNameArr[0]))
						if (serverNameMatcher.find()):
							serverName = serverNameMatcher.group(1)
						#endIf
					else:
						serverName = serverNameArr[1]
					#endIf
					
					if (serverType == 'APPLICATION_SERVER'):
						for processor in self.processors:
							if (isinstance(processor, WasCellProcessor)):
								processor.setIsAdminAgentCell("true")
								processor.serverDiscovered(nodeId, serverId, serverName)
							#endIf
						#endFor
					elif (serverType == 'WEB_SERVER'):
						for processor in self.processors:
							if (isinstance(processor, IhsCellProcessor)):
								self.ihsNodeNum += 1
								self.runManagedNodeWsadminScript(nodeProfileRoot, nodeHost, nodePort, 'RemoteCellProcessor.py', nodeName, serverId, self.ihsNodeNum)
							#endIf
						#endFor
					#endIf
				#endFor
			#endIf
		#endFor
	#endDef

	"""
		Determines the cell type (ND/Stand-alone/AdminAgent/JobManager 
		and invokes appropriate cell processor method
	"""
	def processCell(self):
		self.LOGGER.traceEnter()
		
		## read cell data
		cell = AdminConfig.list("Cell").split(NEWLINE)[0]
		cellName = AdminConfig.showAttribute(cell, "name")
		self.environmentDict["CELL_NAME"] = cellName

		
		##Is it a regular WAS ND/Standalone cell or WAS7.0 type AdminAgent or JobManager type cell
		processType = AdminConfig.showAttribute(AdminConfig.list('Server').split(NEWLINE)[0], "serverType")
		if (processType == 'ADMIN_AGENT'):
			self.processAdminAgentCell(cell)
		else:
			self.processRegularCell(cell)
		#endIf

		
		for processor in self.processors:
			self.environmentDict.update(processor.getVariables())
			if (processor.isAdminAgentCell == "true"):
				if (not isinstance(processor, IhsCellProcessor)):
					self.modifyEnvDictKeys() #Replace all BASE_ with ADMIN_AGENT_
				else:
					self.modifyIHSValues(processor)
				#endIf
			#endIf
		#endFor
		
		self.LOGGER.traceExit()
	#endDef

	"""
		Returns the dictionary which contains the values determined 
		by examining the cell configuration.
	"""
	def getEnvironmentDict(self):
		return self.environmentDict;
	#endDef

	"""
		Writes the dictionary of values to a properties file in alphabetical order.
		This method also modifies strings which represent paths in windows to be escaped for java.
		
		The values are written to <rafwHome>/work/CellDefinitionReader_work.properties
	"""
	def _writeEnvironmentDict(self):
		self.LOGGER.traceEnter()
		rafwHome = Globals.getInstallRoot()
		envPropsFile = rafwHome + "/work/" + "CellDefinitionReader_work" + ".properties"
		## make sure the rafwHome+'/work/' directory exists
		workDir = File(rafwHome + "/work")
		
		if (not workDir.exists()):
			workDir.mkdirs()
		#endIf
		self.LOGGER.log("CRWWA0147I", envPropsFile)
		PropertyFileWriter().writeFile(envPropsFile, JavaConversionHelper.mapDictToJava(self.environmentDict))
		
		ihsPropsFile = File(rafwHome + "/work/" + "RemoteCellProcessor_work" + ".properties")
		if (ihsPropsFile.exists()):
			propUtil = WsadminPropertiesUtil.getInstance(ihsPropsFile)
			propUtil.append(envPropsFile)
			ihsPropsFile.delete()
		#endIf
		
		self.LOGGER.traceExit(envPropsFile)
		return envPropsFile
	#endDef

	"""
		Sets a property on the dictionary of values maintained by this class.
	"""
	def setProperty(self, name, value):
		self.environmentDict[name] = value
	#endDef

	"""
		Processes the cell to determine the configuration settings in the system
		and then writes those discovered values to a properties file.
	"""
	def writeCellProperties(self):
		self.processCell()
		self._writeEnvironmentDict()
	#endDef

	"""
		Called only in case of an administrative agent cell. This method invokes
		the wsadmin process on the managed node and returns the output of the
		command specified to be executed on it.
	"""
	def runManagedNodeWsadminCmd(self, profileRoot, host, port, command, patternString):
		wsadminArgs = " -connType SOAP -lang jython -port " + str(port) + " -host " + str(host)
		if (self.useSoapProps == 'false'):
			wsadminArgs = wsadminArgs + " -username " + str(self.username) + " -password " + str(self.password)
		#endIf
		wsadminArgs = wsadminArgs + " -c \"" + command + "\""
		osname = java.lang.System.getProperty("os.name").lower()
		if (osname.find('windows') != -1):
			cmd = profileRoot + "/bin/wsadmin.bat" + wsadminArgs
			cmds = ["cmd.exe", "/C", cmd]
		else:
			cmd = profileRoot + "/bin/wsadmin.sh" +  wsadminArgs
			cmds = ["/bin/sh", "-c", cmd]
		#endIf
		matchedOutput = []
		processBuilder = java.lang.ProcessBuilder(cmds)
		pattern = java.util.regex.Pattern.compile(patternString)
		
		wasExPattern = java.util.regex.Pattern.compile('WAS.....E')
		admExPattern = java.util.regex.Pattern.compile('ADM.....E')
		
		try:
			process = processBuilder.start()
			reader = java.io.BufferedReader(java.io.InputStreamReader(process.getInputStream()))
			line = reader.readLine()
			
			wasExMatcher = wasExPattern.matcher(java.lang.String(line))
			admExMatcher = admExPattern.matcher(java.lang.String(line))
			
			while (line is not None) :
				if(wasExMatcher.find()):
					raise NameError
				#endIf
				if(admExMatcher.find()):
					raise NameError
				#endIf
				
				matcher = pattern.matcher(java.lang.String (line))
				if (matcher.find()):
					matchedOutput.append(line)
				#endIf
				
				line = reader.readLine()
			#endWhile
		except NameError:
			_Logger("CellDefinitionReader", MessageManager.RB_WEBSPHERE_WAS).log("CRWWA5066E")
			raise MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS, "CRWWA5066E", java.lang.String(host))
		#endTry
		return matchedOutput
	#endDef
	
	"""
		Called only in case of an administrative agent cell. This method invokes
		the wsadmin script on the managed node and creates a properties file in
		the <RAF_HOME>/work directory.
	"""
	def runManagedNodeWsadminScript(self, profileRoot, host, port, scriptName, nodeName, serverId, ihsNodeNum):
		installRoot = Globals.getInstallRoot()
		script =  installRoot + "/product/lib/jython/wsadmin_lib/existingcell/" + scriptName
		
		cmdParams = " -connType SOAP -lang jython -port " + str(port) + " -host " + str(host) 
		if (self.useSoapProps == 'false'):
			cmdParams = cmdParams + " -username " + str(self.username) + " -password " + str(self.password)
		#endIf
		cmdParams = cmdParams + " -wsadmin_classpath " + installRoot + "/product/lib/java/wsadmin_lib/comparator.jar"
		cmdParams = cmdParams + " -javaoption \"-DRAFW_HOME=" + installRoot + "\""
		cmdParams = cmdParams + " -javaoption \"-DWSADMIN_LOG=" + installRoot + "/logs/remoteProcessor-wizard.trace\""
		cmdParams = cmdParams + " -javaoption \"-Dpython.path=" + installRoot + "/product/lib/jython/wsadmin_lib\""
		cmdParams = cmdParams + " -f \"" + str(script) + "\"" + " -ihsNodeNum " + str(ihsNodeNum) + " -nodeName " + str(nodeName )+ " -serverId \"" + str(serverId) + "\""
		
		osname = java.lang.System.getProperty("os.name").lower()
		if (osname.find('windows') != -1):
			cmd = profileRoot + "/bin/wsadmin.bat" + cmdParams
			cmds = ["cmd.exe", "/C", cmd]
		else:
			cmd = profileRoot + "/bin/wsadmin.sh" +  cmdParams
			cmds = ["/bin/sh", "-c", cmd]
		#endIf
		
		wasExPattern = java.util.regex.Pattern.compile('WAS.....E')
		admExPattern = java.util.regex.Pattern.compile('ADM.....E')
		
		processBuilder = java.lang.ProcessBuilder(cmds)
		try:
			process = processBuilder.start()
			reader = java.io.BufferedReader(java.io.InputStreamReader(process.getInputStream()))
			
			line = reader.readLine()
			
			wasExMatcher = wasExPattern.matcher(java.lang.String(line))
			admExMatcher = admExPattern.matcher(java.lang.String(line))
			
			while (line is not None) :
				if(wasExMatcher.find()):
					raise NameError
				#endIf
				if(admExMatcher.find()):
					raise NameError
				#endIf
				line = reader.readLine()
				print line
			#endWhile
		except:
			_Logger("CellDefinitionReader", MessageManager.RB_WEBSPHERE_WAS).log("CRWWA5066E")
			raise MessageManager.getExceptionMessage(MessageManager.RB_WEBSPHERE_WAS, "CRWWA5066E", java.lang.String(host))
		#endTry
	#endDef
	
	"""
		Called only in case of an Admin Agent cell. This method replaces
		all keys in self.environmentDict of the form BASE_ to ADMIN_AGENT_
	"""
	def modifyEnvDictKeys(self):
		baseNodePattern = java.util.regex.Pattern.compile("^BASE_(\\S+)")
		nodePattern = java.util.regex.Pattern.compile("^NODE(\\d+)_(\\S+)")
		
		for key in self.environmentDict.keys():
			val = self.environmentDict[key]
			baseNodeMatcher = baseNodePattern.matcher(java.lang.String(key))
			nodeMatcher = nodePattern.matcher(java.lang.String(key))
			if (baseNodeMatcher.find()):
				subString = baseNodeMatcher.group(1)
				self.environmentDict[self.ADMIN_AGENT_NODE_PREFIX + subString] = val
				del self.environmentDict["BASE_" + subString]
			elif (nodeMatcher.find()):
				nodeNum = nodeMatcher.group(1)
				subString = nodeMatcher.group(2)
				if (nodeNum == '0'):
					del self.environmentDict["NODE" + nodeNum + "_" + subString]
					continue
				#endIf
				self.environmentDict[self.ADMIN_MANAGED_NODE_PREFIX + nodeNum + "_" + subString] = val
			#endIf
		#endFor
	#endDef
	
	"""
		Called only in case of an Admin Agent cell. This method adds IHS
		related keys to the environment dictionary
	"""
	def modifyIHSValues(self, processor):
		rafwHome = Globals.getInstallRoot()
		ihsPropsFile = File(rafwHome + "/work/" + "RemoteCellProcessor_work" + ".properties")
		if (ihsPropsFile.exists()):
			ihsNodes = []
			propUtil = WsadminPropertiesUtil.getInstance(ihsPropsFile)
			for i in range(1, self.ihsNodeNum + 1):
				ihsNodes.append(propUtil.getProperty("IHS_NODE" + str(i) + "_NODE_NAME"))
			#endFor
			self.environmentDict["IHS_NODES"] = ",".join(ihsNodes)
		#endIf
		if (self.ihsNodeNum > 0):
			self.environmentDict["IHS_VERSION"] = propUtil.getProperty("IHS_VERSION")
			self.environmentDict["INSTALL_IHS"] = "yes"
		else:
			self.environmentDict["IHS_VERSION"] = ""
			self.environmentDict["INSTALL_IHS"] = "no"
		#endIf
		self.environmentDict["NUMBER_OF_IHS_NODES"] = str(self.ihsNodeNum)
	#endDef
	
#endClass

## MAIN

optDict, args = SystemUtils.getopt( sys.argv, 'platform:;useSoapProps:;param1:;param2:' )
cdr = CellDefinitionReader()
cdr.useSoapProps = optDict['useSoapProps']
if (cdr.useSoapProps == 'false'):
	cdr.username = optDict['param1']
	cdr.password = optDict['param2']
#endIf

cellProcessorFactory= ProcessorFactory(cdr)
processor = cellProcessorFactory.getProcessorType()
processor.setPlatform(optDict['platform'])
cdr.addProcessor(processor)

ihsProcessor = IhsCellProcessor(cdr)
ihsProcessor.setPlatform(optDict['platform'])
cdr.addProcessor(ihsProcessor)
cdr.writeCellProperties()

