'''
Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Alden Taylor
'''
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from ConfigReader import ConfigReader
from existingcell import Constants

from existingcell.Constants import NEWLINE

import AdminConfig
import AdminTask

"""
	Provides methods to access node configuration information
"""
class Node:
	
	## these must match scope.properties SCOPE_DATA_TYPE property
	MANAGED_NODE="WAS"
	WP_MANAGED_NODE="WP"
	DMGR_NODE="DMGR"
	IHS_NODE="IHS"
	BASE_NODE="WAS_STANDALONE"
	WP_BASE_NODE="WP_STANDALONE"
	ADMIN_AGENT_NODE="ADMIN_AGENT"
	JOB_MANAGER_NODE="JOB_MANAGER"
	ADMIN_MANAGED_NODE="WAS_MANAGED_STANDALONE"
	
	## VariableSubsitutionEntry key names which store paths
	## Values for these keys will the \ replaced with / upon being read 
	VARS_STORING_PATH = ["USER_INSTALL_ROOT", "WAS_INSTALL_ROOT", "WPS_HOME"]
	
	def __init__(self, configId, isAdminAgentCell="false"):
		self.LOGGER = _Logger("Node", MessageManager.RB_WEBSPHERE_WAS)
		self.configId = configId
		self.nodeName = AdminConfig.showAttribute(configId, "name")
		self.wasVersion = ""
		if (isAdminAgentCell == "false"):
			self.wasVersion = self.getNodeVersion()
		self.fullVersion = ""
		self.environmentDict = {}
		self.nodeVars = None
		self.portsByServer = None
		
		self.configReader = ConfigReader()
		self.nodeContainsAppServers = 0
		self.wpPrimaryServer=""
		self.servers = []
		self.wpServers = []
		self.nodeAgents = []
		self.dmgrServer = None
		## default to 0, for unmanaged nodes this is never used
		self.managedNodeNum = 0
		
		## initialize the default type, to be overwritten by discoverServer later
		if(isAdminAgentCell == "true"):
			self.setDataType(Node.ADMIN_AGENT_NODE)
		else:
			self.setDataType(Node.BASE_NODE)
	#endDef
	
	def getDataType(self):
		return self.dataType
	#endDef
	
	def setDataType(self,dtype):
		if( dtype != None):
			self.dataType = dtype
	#endDef
	
	def getWpPrimaryServer(self):
		return self.wpPrimaryServer
	#endDef
	
	def getPrefix(self):
		if(self.dataType == Node.MANAGED_NODE):
			return Constants.NODE + str(self.managedNodeNum)
		elif(self.dataType == Node.DMGR_NODE):
			return Constants.ND
		elif(self.dataType == Node.IHS_NODE):
			return Constants.IHS_NODE
		elif(self.dataType==Node.ADMIN_AGENT_NODE):
			return Constants.ADMIN_AGENT
		elif(self.dataType==Node.ADMIN_MANAGED_NODE):
			return Constants.NODE + str(self.managedNodeNum)
		else:
			return Constants.BASE
	#endDef
			
	def getConfigId(self):
		return self.configId
	#endDef
	
	def getNodeName(self):
		return self.nodeName
	#endDef

	def getNodeNum(self):
		return self.managedNodeNum
	#endDef
	
	"""
		Adds a serverName to the list of WP Servers on this node.
	"""
	def addWPServer(self, serverName):
		self.wpServers.append(serverName)
		if (self.wpPrimaryServer == ""):
			self.wpPrimaryServer = serverName
		#endIf
	#endDef
	
	"""
		Returns the list of wp server names as a comma delimited string
		"wp1,wp2"
		If there are no wp servers on this node, returns ""
	"""
	def getWPServersOnNodeStr(self):
		return ",".join(self.wpServers)
	#endDef
	
	"""
		Adds a serverName to the list of application servers on this node.
	"""
	def addServer(self, server):
		self.servers.append(server)
	#endDef
	
	def getNumServers(self):
		return len(self.servers)
	#endDef
	
	def getNumStandaloneServers(self):
		numServers = 0
		for server in self.servers:
			## must be standalone if its not on a cluster
			if (server.getClusterName() == ""):
				numServers = numServers + 1
			#endIf
		#endFor
		return numServers
	#endDef
	
	def getServers(self):
		return self.servers
	#endDef
	
	def getDmgrServer(self):
		return self.dmgrServer
	#endIf
	
	def setDmgrServer(self, server):
		self.dmgrServer = server
	#endDef
	
	def addNodeAgentServer(self, server):
		self.nodeAgents.append(server)
	#endDef
	
	def getNodeAgentServers(self):
		return self.nodeAgents
	#endDef
	
	"""
		Returns the list of server names as a comma delimited string
		"server1,server2"
		If there are no servers on this node, returns ""
	"""
	def getServersOnNodeStr(self):
		serverNames = []
		for server in self.servers:
			serverNames.append(server.getServerName())
		#endFor
		return ",".join(serverNames)
	#endDef
	
	def getStandaloneServersOnNodeStr(self):
		serverNames = []
		for server in self.servers:
			## must be standalone if its not on a cluster
			if (server.getClusterName() == ""):
				serverNames.append(server.getServerName())
			#endIf
		#endFor
		return ",".join(serverNames)
	#endDef
	
	"""
		Returns the was version as a 2 character string.  [major,minor]
		Example: 61
	"""
	def getNodeVersion(self):
		self.LOGGER.traceEnter()
		productVersion = AdminTask.getNodeBaseProductVersion('[-nodeName ' + self.nodeName + ']')
		major,minor,point = productVersion.split('.', 2)
		version = major + minor
		self.LOGGER.traceExit(version)
		return version
	#endDef
	
	"""
		Returns the full WAS version as in 7.0.0.19
		Example: 8.5.0.0
	"""
	def getNodeFullVersion(self):
		self.LOGGER.traceEnter()
		version = AdminTask.getNodeBaseProductVersion('[-nodeName ' + self.nodeName + ']')
		self.LOGGER.traceExit(version)
		return version
	#endDef
	
	"""
		Returns a dictionary of variables defined at the node scope
		for the supplied node.
		
		Entries defined in VARS_STORING_PATH will have \ replaced with /
	"""
	def getNodeVariables(self):
		self.LOGGER.traceEnter()
		if (self.nodeVars is None):
			self.nodeVars = {}
			for varSub in AdminConfig.list( 'VariableSubstitutionEntry', self.configId ).split( NEWLINE ):
				if (len(varSub) > 0):
					varDict = self.configReader.show(varSub)
					#WAS 60 doesn't return the value attribute if it's blank
					if (not varDict.has_key('value')) :
						varDict['value'] = ""
					#endIf
					key = varDict['symbolicName']
					value = varDict['value']
					## For windows, the Profile Home is returned with a "\". So we convert it before splitting the string
					if (key in Node.VARS_STORING_PATH):
						value = value.replace("\\" , "/")
					#endIf
					self.nodeVars[key] = value 
				#endIf
			#endFor
		#endIf
		self.LOGGER.traceExit(self.nodeVars)
		return self.nodeVars
	#endDef

	"""
		Returns a dictionary of the ports associated with this node.
		
		The returned object is a dictionary keyed by server name, 
		the value is the ports dictionary, keyed by endPointName
		
	"""
	def getPorts(self):
		self.LOGGER.traceEnter()
		if (self.portsByServer is None):
			self.portsByServer = {}
			for serverEntry in AdminConfig.list( 'ServerEntry', self.configId ).split( NEWLINE ):
				if (len (serverEntry) > 0):
					ports = {}
					varDict = self.configReader.showAll(serverEntry)
			
					serverName = varDict["attrs"]["serverName"]
					specialEndpoints = varDict["children"]
					for endPoint in specialEndpoints:
						if( endPoint.has_key("attrs") and endPoint["attrs"].has_key("endPointName") ):
							pointName = endPoint["attrs"]["endPointName"]
						else:
							self.LOGGER.trace("No endPointName, skipping endPoint " + str(endPoint) )
							continue
							
						if( endPoint.has_key("children") ):
							point = endPoint["children"][0]
						else:
							self.LOGGER.trace("No children, skipping endPoint " + str(endPoint) )
							continue
							
						if( point.has_key("attrs") and point["attrs"].has_key("port") ):
							ports[pointName] = point["attrs"]["port"]
						else:
							self.LOGGER.trace("No port, skipping endPoint " + str(endPoint) )
							continue
					#endFor
					self.portsByServer[serverName] = ports
				#endIf
			#endFor
		#endIf
		self.LOGGER.traceExit(self.portsByServer)
		return self.portsByServer
	#endDef
	
	"""
		Returns the profile name denoted by the given path.
		
		Example:
		/opt/ibm/appserver/profiles/dmgr
		returns dmgr
	"""
	def getProfileName(self):
		self.LOGGER.traceEnter()
		if (self.getNodeVariables().has_key('USER_INSTALL_ROOT')):
			profileHome = self.getNodeVariables()['USER_INSTALL_ROOT']
		else:
			raise "VariableSubstitutionEntry with name='USER_INSTALL_ROOT' is not defined in the WebSphere configuration."
		#endIf
		profileHomeSplit = profileHome.split('/')
		profileName = profileHomeSplit[len(profileHomeSplit) -1]
		self.LOGGER.traceExit(profileName)
		return profileName
	#endDef

	""" 
	sets properties about the servers on this node in the supplied hash 
	"""	
	def setServerProperties(self, hash):
		for server in self.getServers():
			serverProps = server.getProperties()
			for key in serverProps.keys():
				hash[key] = serverProps[key]
			#endFor
		#endFor
		## add the dmgr props if available
		if (self.getDmgrServer() is not None):
			serverProps = self.getDmgrServer().getProperties()
			for key in serverProps.keys():
				hash[key] = serverProps[key]
			#endFor
		#endIf
		for server in self.getNodeAgentServers():
			serverProps = server.getProperties()
			for key in serverProps.keys():
				hash[key] = serverProps[key]
			#endFor
		#endFor
	#endDef

#endClass
